@@ -12,6 +12,7 @@ import pandas as pd
 import numpy as np
 import dropbox
 from pathlib import Path
+from io import StringIO
 
 @contextlib.contextmanager
 def stopwatch(message):
@@ -30,6 +31,18 @@ class NaPoleonDropboxConnector(object):
         self.overwrite = overwrite
         self.dropbox_backup = dropbox_backup
 
+    def download_csv(self, csv_file_name = '', sep = ',', index_date = False):
+        bytes_data_csv = self.download('', '', csv_file_name)
+        string_data_csv = StringIO(str(bytes_data_csv, 'utf-8'))
+        dataframe = pd.read_csv(string_data_csv, sep=sep)
+        if index_date:
+            dataframe = dataframe.rename(columns={"Unnamed: 0": "date"}, errors="raise")
+            dataframe['date'] = pd.to_datetime(dataframe['date'])
+            dataframe = dataframe.sort_values(by=['date'])
+            dataframe = dataframe.set_index(dataframe['date'])
+            dataframe = dataframe.drop(columns=['date'])
+        return dataframe
+
     def uploadFileToDropbox(self, filename='', dropbox_subfolder = '', fullpath=''):
         self.upload(fullname=fullpath, folder='', subfolder=dropbox_subfolder, name=filename, overwrite=self.overwrite)
 
