@@ -5,6 +5,7 @@ import sqlite3
 
 from multiprocessing import Pool
 from napoleontoolbox.file_saver import dropbox_file_saver
+from napoleontoolbox.parallel_run import launcher_utility
 
 from napoleontoolbox.signal import signal_utility
 import json
@@ -14,7 +15,9 @@ import math
 import numpy as np
 
 class ForecasterParalleLauncher():
-    def __init__(self,drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon',  db_path_suffix = '_run.sqlite', ss=[10], ns=[21], s_evals = [5], calibration_steps=[1000], optimize_thresholds=[True], signal_idiosyncratic_paramaters=[None], table_max_size = 2000):
+    def __init__(self,starting_date = None, running_date = None, drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon', underlying=None,frequence=None,algo=None,db_path_suffix = '_forecaster.sqlite', ss=[10], ns=[21], s_evals = [5], calibration_steps=[1000], optimize_thresholds=[True], signal_idiosyncratic_paramaters=[None], table_max_size = 2000):
+        self.starting_date = starting_date
+        self.running_date = running_date
         self.args = []
         self.key_args=[]
         self.counter = 1
@@ -43,8 +46,9 @@ class ForecasterParalleLauncher():
         self.run_number = self.counter - 1
         self.local_root_directory = local_root_directory
         self.user = user
+
         self.db_path_suffix = db_path_suffix
-        self.filename =  user + db_path_suffix
+        self.filename =  launcher_utility.get_database_name(user, frequence, underlying, algo, starting_date, running_date, db_path_suffix)
         self.db_path = self.local_root_directory + self.filename
         self.runs = []
         self.totalRow = 0
