@@ -16,7 +16,7 @@ import json
 from pathlib import Path
 
 class AbstractRunner(ABC):
-    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, underlying = None, frequence='daily', algo='' ,freqly_return_pkl_filename_suffix='freqly_candels.pkl',aggregated_pkl_file_suffix='freqly_to_mix.pkl', aggregated_pkl_mapping_file_suffix = 'freqly_to_mix_mapping.pkl', list_pkl_file_suffix = 'my_list.pkl', local_root_directory='../data/', user = 'napoleon', add_turn_over = False, skipping_size=200, recompute_all = True, db_path_suffix=None):
+    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, underlying = None, frequence='daily', selected_algo='',algo='' ,freqly_return_pkl_filename_suffix='freqly_candels.pkl',aggregated_pkl_file_suffix='freqly_to_mix.pkl', aggregated_pkl_mapping_file_suffix = 'freqly_to_mix_mapping.pkl', list_pkl_file_suffix = 'my_list.pkl', local_root_directory='../data/', user = 'napoleon', add_turn_over = False, skipping_size=200, recompute_all = True, db_path_suffix=None):
         super().__init__()
         self.freqly_return_pkl_filename_suffix=freqly_return_pkl_filename_suffix
         self.aggregated_pkl_file_suffix=aggregated_pkl_file_suffix
@@ -25,10 +25,10 @@ class AbstractRunner(ABC):
         self.list_pkl_file_suffix = list_pkl_file_suffix
         self.frequence=frequence
         self.algo=algo
-
-        self.list_pkl_file_name = self.underlying + '_' + self.frequence + '_' + self.algo + self.list_pkl_file_suffix
-        self.aggregated_pkl_file_name = self.underlying + '_' + self.frequence + '_' + self.algo + self.aggregated_pkl_file_suffix
-        self.aggregated_pkl_mapping_file_name = self.underlying + '_' + self.frequence + '_' + self.algo + self.aggregated_pkl_mapping_file_suffix
+        self.selected_algo=selected_algo
+        self.list_pkl_file_name = self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.list_pkl_file_suffix
+        self.aggregated_pkl_file_name = self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.aggregated_pkl_file_suffix
+        self.aggregated_pkl_mapping_file_name = self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.aggregated_pkl_mapping_file_suffix
 
         self.local_root_directory=local_root_directory
         self.user=user
@@ -48,7 +48,7 @@ class AbstractRunner(ABC):
         if not self.recompute_all:
             saver = signal_result_analyzer.ParallelRunResultAnalyzer(drop_token=drop_token,
                                                                      local_root_directory=local_root_directory, user=user,
-                                                                     db_path_suffix=self.db_path_name)
+                                                                     db_path_name=self.db_path_name)
             self.precomputed_results_df, self.precomputed_signals_df = saver.analyzeAllRunResults()
         else:
             self.precomputed_results_df, self.precomputed_signals_df  = None, None
